package com.ozs.web.controller.accountmanagment;


import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.utils.AppendUtils;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.BaseRailwayManagement;
import com.ozs.service.entity.MsgAlarm;
import com.ozs.service.entity.MsgAlarmFrequency;
import com.ozs.service.entity.vo.MsgAlarmResp;
import com.ozs.service.entity.vo.MsgAlarmVo;
import com.ozs.service.service.*;
import com.ozs.web.controller.websocket.WebSocketServer;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 报警信息表 前端控制器
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@RestController
@RequestMapping("/service/msgAlarm")
public class MsgAlarmController extends BaseController {
    @Resource
    MsgAlarmService msgAlarmService;
    @Resource
    MsgAlarmFrequencyService msgAlarmFrequencyService;
    @Autowired
    BaseCameraManagementService baseCameraManagementService;
    @Autowired
    BaseRailwayManagementService baseRailwayManagementService;
    @Value("${base.imgUrl:http://124.71.171.71:18801/picbucket}")
    private String imgUrl;
    @Autowired
    private BaseUserService baseUserService;
    @Autowired
    private WebSocketServer webSocketServer;

    /**
     * 测试消息推送
     */
    @PostMapping("/push")
    @ApiOperation(value = "预报警信息分页")
    public AjaxResult insertDate(String args) {
        String userId = getUserId();
        // 初始化 -> 运行中 -> 成功或失败
        try {
            // 1、初始化
            webSocketServer.sendOneMessage(userId, objStr(args, 1));
            Thread.sleep(3 * 1000);
            // 2、运行中
            webSocketServer.sendOneMessage(userId, objStr(args, 2));
            Thread.sleep(5 * 1000);
            // 3、成功或失败
            webSocketServer.sendOneMessage(userId, Math.random() * 10 > 5 ? objStr(args, 6) : objStr(args, 7));
        } catch (InterruptedException e) {
            e.printStackTrace();
            throw new RuntimeException("构建失败");
        }
        return AjaxResult.success("推送成功");
    }

    private String objStr(String str, int status) {
        JSONObject obj = new JSONObject();
        obj.put("pipelineId", str); //流水线ID
        obj.put("status", status);
        return obj.toJSONString();
    }

    /**
     * 报警信息分页
     *
     * @param msgAlarmVo
     * @return
     */
    @ApiOperation(value = "报警信息分页")
    @PostMapping("/list")
    public AjaxResult selectMsgAlarm(@RequestBody MsgAlarmVo msgAlarmVo) {
        List<BaseCameraManagement> baseCameraManagementList = new ArrayList<>();
        LambdaQueryWrapper<BaseCameraManagement> wrapper = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(msgAlarmVo.getRailwayCode())) {
            wrapper.eq(BaseCameraManagement::getRailwayCode, msgAlarmVo.getRailwayCode());
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getBeginMile())) {
            wrapper.ge(BaseCameraManagement::getBeginMile, msgAlarmVo.getBeginMile());
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getEndMile())) {
            wrapper.le(BaseCameraManagement::getEndMile, msgAlarmVo.getEndMile());
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getRailwayCode()) || !ObjectUtils.isEmpty(msgAlarmVo.getBeginMile()) || !ObjectUtils.isEmpty(msgAlarmVo.getEndMile())) {
            baseCameraManagementList = baseCameraManagementService.list(wrapper);
        }
        List<String> userIds = baseCameraManagementService.getUserIds(getUserId());
        LambdaQueryWrapper<MsgAlarm> lw = new LambdaQueryWrapper<MsgAlarm>();
        if (!ObjectUtils.isEmpty(userIds)) {
            lw.in(MsgAlarm::getCreateBy, userIds);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getLineDir())) {
            lw.eq(MsgAlarm::getLineDir, msgAlarmVo.getLineDir());
        }
        if (baseCameraManagementList.size() > 0) {
            List<String> collect = baseCameraManagementList.stream().map(BaseCameraManagement::getCameraCode).collect(Collectors.toList());
            lw.in(MsgAlarm::getCameraCode, collect);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getAlarmType())) {
            lw.eq(MsgAlarm::getAlarmType, msgAlarmVo.getAlarmType());
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getAlarmMile()) && msgAlarmVo.getAlarmMile() != 0) {
            lw.eq(MsgAlarm::getAlarmMile, msgAlarmVo.getAlarmMile());
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getBeginAlarmTime())) {
            lw.ge(MsgAlarm::getAlarmTime, msgAlarmVo.getBeginAlarmTime());
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getEndAlarmTime())) {
            lw.le(MsgAlarm::getAlarmTime, msgAlarmVo.getEndAlarmTime());
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getIsLock())) {
            lw.eq(MsgAlarm::getIsLock, msgAlarmVo.getIsLock());
        }
        IPage<MsgAlarm> page = msgAlarmService.page(new Page<>(msgAlarmVo.getPageNum(), msgAlarmVo.getPageSize()), lw);
        if (!ObjectUtils.isEmpty(page) && !ObjectUtils.isEmpty(page.getRecords())) {
            List<MsgAlarm> dto1 = page.getRecords().stream().map(o -> {
                ArrayList<String> objects = new ArrayList<>();
                String[] split = o.getImageUrl().split(";");
                for (String s : split) {
                    s = imgUrl + s;
                    objects.add(s);
                }
                o.setImageUrls(objects);
                LambdaQueryWrapper<BaseCameraManagement> cameraManagementLambdaQueryWrapper = new LambdaQueryWrapper<BaseCameraManagement>();
                if (!ObjectUtils.isEmpty(o.getCameraCode())) {
                    cameraManagementLambdaQueryWrapper.eq(BaseCameraManagement::getCameraCode, o.getCameraCode());
                }
                BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(cameraManagementLambdaQueryWrapper);
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(cameraManagement.getRailwayCode())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, cameraManagement.getRailwayCode());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);

                o.setRailwayName(baseRailwayManagement.getRailwayName());
                String mils = AppendUtils.stringAppend(cameraManagement.getInstallMile());
                o.setAlarmMiles(mils);
                String end = AppendUtils.stringAppend(cameraManagement.getEndMile());
                o.setEndMile(end);
                String begin = AppendUtils.stringAppend(cameraManagement.getBeginMile());
                o.setBeginMile(begin);
                o.setAlarmCount((int) msgAlarmFrequencyService.count() + 1);
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }

    @GetMapping(value = "/{id}")
    @ApiOperation("根据报警信息ID获取详细信息")
    public AjaxResult getInfo(@PathVariable Long id) {
        MsgAlarm msgAlarm = msgAlarmService.getById(id);
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
            lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
        }
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
        LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
            wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
        MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
        BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
        msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
        msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
        String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
        String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
        String endMile = begin + "-" + end;
        msgAlarmResp.setBeginMile(endMile);
        String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
        msgAlarmResp.setAlarmMile(mile);
        msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
        return AjaxResult.success(msgAlarmResp);
    }


    @PutMapping(value = "/{id}")
    @ApiOperation("根据报警信息ID解除报警")
    public AjaxResult updateIsLock(@PathVariable Long id) {
        MsgAlarm msgAlarm = msgAlarmService.getById(id);
        msgAlarm.setIsLock(1);
        msgAlarm.setUpdateBy(getUserId());
        msgAlarm.setReleasedBy(getUserId());
        msgAlarm.setReleasedTime(new Date());
        return AjaxResult.success(msgAlarmService.updateById(msgAlarm));
    }

    @GetMapping(value = "/alarmHistoricalAlarm/{alarmId}")
    @ApiOperation("报警消息详情历史报警次数信息")
    public AjaxResult alarmHistoricalAlarm(@PathVariable Long alarmId) {
        QueryWrapper<MsgAlarmFrequency> wrapper = new QueryWrapper<>();
        wrapper.eq("alarm_id", alarmId);
        wrapper.orderByDesc("alarm_time");
        List<MsgAlarmFrequency> list = msgAlarmFrequencyService.list(wrapper);
        return AjaxResult.success(list);
    }

    @GetMapping(value = "/videoHistoricalAlarm/{cameraCode}")
    @ApiOperation("视频服务历史报警信息")
    public AjaxResult videoHistoricalAlarm(@PathVariable Long cameraCode) {
        QueryWrapper<MsgAlarm> wrapper = new QueryWrapper<>();
        wrapper.eq("camera_code", cameraCode);
        wrapper.orderByDesc("alarm_time");
        List<MsgAlarm> list = msgAlarmService.list(wrapper);
        return AjaxResult.success(list);
    }
}

